@@ -28,17 +28,38 @@ from collections import Counter
 proton = 1.007276466879
 electron = 0.000548579909070
 
-ionising_species = { # '<element+charge>': [formula: Counter, charge: int, mass_correction: float]
-    'none':   [Counter(), 1, 0],
-    '+':   [Counter(), 1, - electron],
-    'H+':   [Counter(dict(H=1)), 1, - electron],
-    'NH4+': [Counter(dict(N=1, H=4)), 1, - electron],
-    'Na+':  [Counter(dict(Na=1)), 1, - electron],
-    'K+':   [Counter(dict(K=1)), 1, - electron],
-    '-':    [Counter(), -1, electron],
-    '-H+':  [Counter(dict(H=-1)), -1, electron],
-    'Cl-':   [Counter(dict(Cl=1)), -1, electron],
-    'I-':   [Counter(dict(I=1)), -1, electron],
+ionising_species = { # '<element+charge>': [formula: Counter, charge: int, mass_correction: str]
+    # Mass correction uses the virtual elements E for +electron or Me for -electron
+    'none': [Counter(),
+             1,
+             ''],
+    '+':    [Counter(),
+             1,
+             'Me'],
+    'H+':   [Counter(dict(H=1)),
+             1,
+             'Me'],
+    'NH4+': [Counter(dict(N=1, H=4)),
+             1,
+             'Me'],
+    'Na+':  [Counter(dict(Na=1)),
+             1,
+             'Me'],
+    'K+':   [Counter(dict(K=1)),
+             1,
+             'Me'],
+    '-':    [Counter(),
+             -1,
+             'E'],
+    '-H+':  [Counter(dict(H=-1)),
+             -1,
+             'E'],
+    'Cl-':  [Counter(dict(Cl=1)),
+             -1,
+             'E'],
+    'I-':   [Counter(dict(I=1)),
+             -1,
+             'E'],
 }
 
 monomers = {
