@@ -70,41 +70,8 @@ def toRelativeAbundance(peaklist: list):
     maxAbund = max(list(zip(*peaklist))[1])
     return sorted(list((mass, normAbund/maxAbund) for mass, normAbund in peaklist))
 
-def combineIsotopes_old(peaklist: list, resolution: float):
-    # returns a list of peaks [(mz, normAbundance)]
-    if type(peaklist) == list:
-        peaklist_df = pd.DataFrame(peaklist, columns=["mz", "abundance"])
-    else:
-        columns_temp = peaklist.columns
-        peaklist_df = peaklist
-        peaklist_df.columns = ["mz", "abundance"]
-
-    index = 0
-    while index < len(peaklist_df):
-        if not peaklist_df.iloc[index].isnull().all():  # checks that 'not all values in a row are NaN', alternative: .any()
-            mz = peaklist_df.iloc[index].mz
-            # stdDev = mz / (2 * sqrt(2 * log(2)) * resolution); +- 2stdDev contains 95% of the normal distribution while +- 1.17741stdDev is the FWHM range
-            dmz = mz / (2 * resolution)
-            masses = list(peaklist_df.loc[(peaklist_df.mz >= mz - dmz) & (peaklist_df.mz <= mz + dmz)].mz)
-            if len(masses) > 1:
-                abundances = list(peaklist_df.loc[(peaklist_df.mz >= mz - dmz) & (peaklist_df.mz <= mz + dmz)].abundance)
-                peaklist_df.loc[(peaklist_df.mz >= mz - dmz) & (peaklist_df.mz <= mz + dmz)] = None
-                peaklist_df.iloc[index] = np.average(masses, weights=abundances), sum(abundances)
-        index += 1
-    peaklist_df.dropna(how="all", inplace=True)  # get rid of all the empty lines, how=all limits to rows that are completely empty
-    peaklist_df.reset_index(drop=True, inplace=True)
-    # peaklist_df.normAbundance = peaklist_df.normAbundance / peaklist_df.normAbundance.sum()  # recalculate normalised abundance
-    # if the sum = 1 for the input (normAbundance), then the sum will still be 1 when simply adding peaks together.
-    # if the input is no normAbundance, then the recalculation has to be done externally
-
-    if type(peaklist) == list:
-        return list(peaklist_df.itertuples(index=False, name=None))
-    else:
-        peaklist_df.columns = columns_temp
-        return peaklist_df
-
 # @jit(nopython=True)
-def combineIsotopes(peaklist: list, resolution: float):
+def combineIsotopes_old(peaklist: list, resolution: float):
     # returns a list of peaks [(mz, normAbundance)]
     if type(peaklist) == list:
         peaklist_df = pd.DataFrame(peaklist, columns=["mz", "abundance"])
@@ -133,37 +100,31 @@ def combineIsotopes(peaklist: list, resolution: float):
     else:
         return pd.DataFrame(result, columns=columns_temp)
 
-'''
-def combineIsotopes2(peaklist: list, resolution: float):
+# @jit(nopython=True)
+def combineIsotopes(peaklist: list, resolution: float):
     # returns a list of peaks [(mz, normAbundance)]
     if type(peaklist) == list:
-        peaklist_df = pd.DataFrame(peaklist, columns=["mz", "abundance"])
-        columns_temp = ["mz", "abundance"]
+        list_out = True
+        peaklist = np.array(peaklist)
     else:
-        columns_temp = peaklist.columns
-        peaklist_df = peaklist.copy(deep=true)
-        peaklist_df.columns = ["mz", "abundance"]
+        list_out = False
 
-    peaklist_df.sort_values(by=['abundance'], ascending=False, inplace=True, ignore_index=True)
-    result = pd.DataFrame(columns=columns_temp)
-    while len(peaklist_df) > 0:
-        mz = peaklist_df.mz.iloc[0]
-        dmz = mz / (2 * resolution)
-        indices = peaklist_df.loc[(peaklist_df.mz >= mz - dmz) & (peaklist_df.mz <= mz + dmz)].index
-        extract = peaklist_df.loc[indices]
-        peaklist_df.drop(indices, inplace=True)
-        if len(extract) > 1:
-            result = result.append(pd.DataFrame([np.average(extract.mz, weights=extract.abundance, returned=True)], columns=columns_temp), ignore_index=True)
-        else:
-            result = result.append(extract, ignore_index=True)
-    result.sort_values(by=['mz'], ascending=True, inplace=True, ignore_index=True)
+    # sort in descending peak height
+    peaklist = peaklist[peaklist[:, 1].argsort()[::-1]]
+    result = np.array([])
+    while len(peaklist) > 0:
+        mz = peaklist[0, 0]
+        dmz = 1.4 * mz / resolution # was mz / (2 * resolution) in previous versions
+        # FWHM: dm = m/R --> peak = m +- m/(2R); 10% valley (95% normal): dm approx 2m/R --> peak = m +- m/R; 99.5% normal: dm approx 3m/R --> peak = m +- 1.5m/R
+        # factor 1.4 estimated from simulations in Xcalibur with regard to centroiding and resolution
+        indices = np.where(np.logical_and(peaklist[:, 0] >= mz - dmz, peaklist[:, 0] <= mz + dmz))
+        extract = peaklist[indices]
+        result = np.append(result, np.average(extract[:, 0], weights=extract[:, 1], returned=True))
+        peaklist = np.delete(peaklist, indices, axis=0)
 
-    if type(peaklist) == list:
-        return list(result.itertuples(index=False, name=None))
-    else:
-        result.columns = columns_temp
-        return result
-'''
+    result = np.reshape(result, (-1, 2))
+    result = result[result[:, 0].argsort()] # sort mz
+    return [(mz, abund) for mz, abund in result] if list_out else result
 
 def gui():
     ThermoRawFrame = [
