@@ -32,44 +32,80 @@ from operator import itemgetter, attrgetter
 import re
 import sys
 import progressbar
+import copy
+
+# todo: remove dependency on self.isobaricSpecies as the same info is now available under self.macromolecules directly as 'isobaricCompositions'
 
 class Polymer:
 
     @staticmethod
-    def getRUCombinations(minRepeatingUnits, maxRepeatingUnits, monomersAmount: int):
+    def getRUCombinations(monomersAmount: int,
+                          minRepeatingUnits: list = None, minSumRepeatingUnits: int = None,
+                          maxRepeatingUnits: list = None, maxSumRepeatingUnits: int = None):
+        #process inputs
+        if minRepeatingUnits is None and minSumRepeatingUnits is None:
+            raise ValueError('Either minRepeatingUnits or minSumRepeatingUnits must not be None')
+        elif minRepeatingUnits is None:
+            minRepeatingUnits = minSumRepeatingUnits
+        elif minSumRepeatingUnits is None:
+            minSumRepeatingUnits = sum(minRepeatingUnits) if type(minRepeatingUnits) == list else minRepeatingUnits
+        if maxRepeatingUnits is None and maxSumRepeatingUnits is None:
+            raise ValueError('Either maxRepeatingUnits or maxSumRepeatingUnits must not be None')
+        elif maxRepeatingUnits is None:
+            maxRepeatingUnits = maxSumRepeatingUnits
+        elif maxSumRepeatingUnits is None:
+            maxSumRepeatingUnits = sum(maxRepeatingUnits) if type(maxRepeatingUnits) == list else maxRepeatingUnits
+
+        assert maxSumRepeatingUnits >= minSumRepeatingUnits, "With the current parameters, minSumRepeatingUnits is higher than maxSumRepeatingUnits, this can't be correct..."
+        # if lists are passed as either min or maxRepeatingUnits, their length must match the amount of monomers
+        if type(minRepeatingUnits) == list and type(maxRepeatingUnits) == list:
+            assert sum(maxRepeatingUnits) >= sum(minRepeatingUnits), "The sum of minRepeatingUnits is higher than the sum of maxRepeatingUnits, this can't be correct..."
+            assert len(minRepeatingUnits) == monomersAmount, "The length of minRepeatingUnits does not match the amount of monomers!"
+            assert len(maxRepeatingUnits) == monomersAmount, "The length of maxRepeatingUnits does not match the amount of monomers!"
+        else:
+            if type(minRepeatingUnits) == list:
+                assert maxRepeatingUnits >= sum(minRepeatingUnits), "The sum of minRepeatingUnits is higher than maxRepeatingUnits, this can't be correct..."
+                assert len(minRepeatingUnits) == monomersAmount, "The length of minRepeatingUnits does not match the amount of monomers!"
+            if type(maxRepeatingUnits) == list:
+                assert sum(maxRepeatingUnits) >= minRepeatingUnits, "minRepeatingUnits is higher than the sum of maxRepeatingUnits, this can't be correct..."
+                assert len(maxRepeatingUnits) == monomersAmount, "The length of maxRepeatingUnits does not match the amount of monomers!"
+            else:
+                assert maxRepeatingUnits >= minRepeatingUnits, "minRepeatingUnits is higher than maxRepeatingUnits, this can't be correct..."
+
+        # Now assemble the matrix
         if type(minRepeatingUnits) == list:
             if type(maxRepeatingUnits) == list:
-                # first check that the length of the list is valid
-                if not len(minRepeatingUnits) == len(maxRepeatingUnits) == monomersAmount:
-                    sys.exit("The length of minRepeatingUnits or maxRepeatingUnits does not match the amount of monomers!\nEither provide a max for each monomer or 1 global max for the sum of the repeating units")
                 RUCombinations = [(a,) for a in range(minRepeatingUnits[0], maxRepeatingUnits[0] + 1)]
                 for i in range(1, monomersAmount):
                     RUCombinations = [(*a, b) for a in RUCombinations for b in
-                                      range(minRepeatingUnits[i], maxRepeatingUnits[i] + 1)]
-                return RUCombinations
+                                      range(minRepeatingUnits[i], maxRepeatingUnits[i] + 1)
+                                      if sum(a) + b <= maxSumRepeatingUnits]
+                if minSumRepeatingUnits > sum(minRepeatingUnits):
+                    RUCombinations = [RUCombination for RUCombination in RUCombinations if
+                                      sum(RUCombination) >= minSumRepeatingUnits]
+                return None if len(RUCombinations) == 0 else RUCombinations
             elif type(maxRepeatingUnits) == int:
-                # first check that the length of the list is valid
-                if not len(minRepeatingUnits) == monomersAmount:
-                    sys.exit("The length of minRepeatingUnits does not match the amount of monomers!\nEither provide a max for each monomer or 1 global max for the sum of the repeating units")
                 RUCombinations = [(a,) for a in range(minRepeatingUnits[0], maxRepeatingUnits + 1)]
                 for i in range(1, monomersAmount):
                     RUCombinations = [(*a, b) for a in RUCombinations for b in
                                       range(minRepeatingUnits[i], maxRepeatingUnits + 1 - sum(a))]
-                return RUCombinations
+                if minSumRepeatingUnits > sum(minRepeatingUnits):
+                    RUCombinations = [RUCombination for RUCombination in RUCombinations if
+                                      sum(RUCombination) >= minSumRepeatingUnits]
+                return None if len(RUCombinations) == 0 else RUCombinations
             else:
-                sys.exit("maxRepeatingUnits must be either int or list.")
+                raise ValueError("maxRepeatingUnits must be either int or list.")
         elif type(minRepeatingUnits) == int:
             # in this case, only a global minimum is given, so all combinations have to be made from 0
-            # the ones with a sum smaller than minRepeatingUnits will be filtered later
+            # the ones with a sum smaller than minSumRepeatingUnits will be filtered later
             if type(maxRepeatingUnits) == list:
-                # first check that the length of the list is valid
-                if not len(maxRepeatingUnits) == monomersAmount:
-                    sys.exit("The length of maxRepeatingUnits does not match the amount of monomers!\nEither provide a max for each monomer or 1 global max for the sum of the repeating units")
                 RUCombinations = [(a,) for a in range(maxRepeatingUnits[0] + 1)]
                 for i in range(1, monomersAmount):
-                    RUCombinations = [(*a, b) for a in RUCombinations for b in range(maxRepeatingUnits[i] + 1)]
+                    RUCombinations = [(*a, b) for a in RUCombinations
+                                      for b in range(maxRepeatingUnits[i] + 1)
+                                      if sum(a) + b <= maxSumRepeatingUnits]
                 RUCombinations = [RUCombination for RUCombination in RUCombinations if
-                                  sum(RUCombination) >= minRepeatingUnits]
+                                  sum(RUCombination) >= minSumRepeatingUnits]
                 return None if len(RUCombinations) == 0 else RUCombinations
             elif type(maxRepeatingUnits) == int:
                 # most simple case: every monomer from 0 to the maxRepeatingUnits minus what's already there from previous iterations
@@ -77,12 +113,12 @@ class Polymer:
                 for i in range(1, monomersAmount):
                     RUCombinations = [(*a, b) for a in RUCombinations for b in range(maxRepeatingUnits + 1 - sum(a))]
                 RUCombinations = [RUCombination for RUCombination in RUCombinations if
-                                  sum(RUCombination) >= minRepeatingUnits]
+                                  sum(RUCombination) >= minSumRepeatingUnits]
                 return None if len(RUCombinations) == 0 else RUCombinations
             else:
-                sys.exit("maxRepeatingUnits must be either int or list.")
+                raise ValueError("maxRepeatingUnits must be either int or list.")
         else:
-            sys.exit("minRepeatingUnits must be either int or list.")
+            raise ValueError("minRepeatingUnits must be either int or list.")
 
     def __init__(self,
                  endgroupPairs,
@@ -91,8 +127,11 @@ class Polymer:
                  charge = 1,
                  minRelAbundance = 0.01,
                  minRepeatingUnits = None,
+                 minSumRepeatingUnits=None,
                  maxRepeatingUnits = None,
+                 maxSumRepeatingUnits=None,
                  mzRange = None,
+                 compositions = None,
                  customEndgroupsDatabase = None,
                  customMonomersDatabase = None,
                  firstMonomerMajor: bool = False,
@@ -105,9 +144,12 @@ class Polymer:
         :type adductIon: str
         :type charge: int
         :type minRelAbundance: float
-        :type minRepeatingUnits: int or list
-        :type maxRepeatingUnits: int or list
+        :type minRepeatingUnits: list
+        :type minSumRepeatingUnits: int
+        :type maxRepeatingUnits: list
+        :type maxSumRepeatingUnits: int
         :type mzRange: list
+        :type compositions: list
         :type customEndgroupsDatabase: dict
         :type customMonomersDatabase: dict
         """
@@ -125,6 +167,9 @@ class Polymer:
             self.customMonomersDatabase = customMonomersDatabase
         else:
             self.customMonomersDatabase = dict()
+        # Workaround for inheritance issues
+        endgroupPairs = copy.copy(endgroupPairs)
+        monomers = copy.copy(monomers)
         print("Simulating polymer...")
         if type(endgroupPairs) != list:
             endgroupPairs = [endgroupPairs]
@@ -165,52 +210,58 @@ class Polymer:
         except:
             sys.exit("adductIon not found in database!")
         self.amountAdductIons = abs(int(charge / adductIon[1]))
-        # macromolecules will be generated with a defined minimum and maximum (total) number of repeating units
-        # These boundaries can be given as arguments, but can also be estimated from an mzRange and
-        # the molecular weights of the smallest (maxRU) and biggest (minRU) monomer:
-        if mzRange is None and maxRepeatingUnits is None:
-            # it will be impossible to estimate the range of repeating units
-            sys.exit("I'm struggling to build a matrix of possible combinations of monomers based on \nthe provided combination of mzRange, minRepeatingUnits and maxRepeatingUnits.\nPlease check the parameters.")
-        elif maxRepeatingUnits is None:
-            if minRepeatingUnits is None:
-                minRepeatingUnits = max(int((mzRange[0] * abs(charge) - max(list(pymacroms.getMonoIsotopicMass(endgroupPair) for endgroupPair in endgroupPairs)) - abs(int(charge / adductIon[1])) * pymacroms.getMonoIsotopicMass(adductIon[0])) / max(list(pymacroms.getMonoIsotopicMass(monomer) for monomer in monomers))), 0)
-                print("--> minimum sum of repeating units: %i (estimated based on the provided mzRange and the masses of the components)" % minRepeatingUnits)
-            maxRepeatingUnits = int((mzRange[1] * abs(charge) - min(list(pymacroms.getMonoIsotopicMass(endgroupPair) for endgroupPair in endgroupPairs)) - abs(int(charge / adductIon[1])) * pymacroms.getMonoIsotopicMass(adductIon[0])) / min(list(pymacroms.getMonoIsotopicMass(monomer) for monomer in monomers))) + 1
-            print("--> maximum sum of repeating units: %i (estimated based on the provided mzRange and the masses of the components)" % maxRepeatingUnits)
-        elif minRepeatingUnits is None:
-            if mzRange is not None:
-                minRepeatingUnits = max(int((mzRange[0] * abs(charge) - max(list(pymacroms.getMonoIsotopicMass(endgroupPair) for endgroupPair in endgroupPairs)) - abs(int(charge / adductIon[1])) * pymacroms.getMonoIsotopicMass(adductIon[0])) / max(list(pymacroms.getMonoIsotopicMass(monomer) for monomer in monomers))), 0)
-                print("--> minimum sum of repeating units: %i (estimated based on the provided mzRange and the masses of the components)" % minRepeatingUnits)
-            else:
-                minRepeatingUnits = 0
-                print("--> minimum sum of repeating units set to 0 since it cannot be estimated")
-        # Now build all the combinations of repeating units within the limits of the given (or estimated) parameters
-        RUCombinations = self.getRUCombinations(minRepeatingUnits, maxRepeatingUnits, len(monomers))
-        if RUCombinations is None:
-            sys.exit("Failed to build a matrix of possible combinations of monomers based on \nthe provided combination of mzRange, minRepeatingUnits and maxRepeatingUnits.\nPlease check the parameters.")
-        if type(minRepeatingUnits) == list:
-            minRepeatingUnits = sum(minRepeatingUnits)
-        if type(maxRepeatingUnits) == list:
-            maxRepeatingUnits = sum(maxRepeatingUnits)
-        if minRepeatingUnits > maxRepeatingUnits:
-            sys.exit("With the current parameters, minRepeatingUnits is higher than maxRepeatingUnits, this can't be correct...")
-        # Finally, combine the combinations of repeating units with the different indices of the different end groups
-        compositions = itertools.product(range(len(endgroupPairs)), RUCombinations)
-        # Dropped the conversion from itertools.product to list as this gave a serious computational penalty and later it doesn't matter for the iteration
-        # itertools doesn't have a length attribute though so output had to be adjusted as well
-        # print("--> Possible combinations of repeating units/end groups based on given parameters: " + str(len(compositions)))
-        print("--> Possible combinations of repeating units/end groups determined")
+        if compositions is None:
+            # macromolecules will be generated with a defined minimum and maximum (total) number of repeating units
+            # These boundaries can be given as arguments, but can also be estimated from an mzRange and
+            # the molecular weights of the smallest (maxRU) and biggest (minRU) monomer:
+            # But first compatibility with old code:
+            if minSumRepeatingUnits is None and type(minRepeatingUnits) == int:
+                minSumRepeatingUnits = minRepeatingUnits
+            if maxSumRepeatingUnits is None and type(maxRepeatingUnits) == int:
+                maxSumRepeatingUnits = maxRepeatingUnits
+            if maxRepeatingUnits is None and maxSumRepeatingUnits is None and mzRange is None:
+                # it will be impossible to estimate the range of repeating units
+                sys.exit("I'm struggling to build a matrix of possible combinations of monomers based on \nthe provided combination of mzRange, min(Sum)RepeatingUnits and max(Sum)RepeatingUnits.\nPlease check the parameters.")
+            elif maxSumRepeatingUnits is None and mzRange is not None:
+                if minSumRepeatingUnits is None:
+                    minSumRepeatingUnits = max(int((mzRange[0] * abs(charge) - max(list(pymacroms.getMonoIsotopicMass(endgroupPair) for endgroupPair in endgroupPairs)) - abs(int(charge / adductIon[1])) * pymacroms.getMonoIsotopicMass(adductIon[0])) / max(list(pymacroms.getMonoIsotopicMass(monomer) for monomer in monomers))), 0)
+                    print("--> minimum sum of repeating units: %i (estimated based on the provided mzRange and the masses of the components)" % minSumRepeatingUnits)
+                maxSumRepeatingUnits = int((mzRange[1] * abs(charge) - min(list(pymacroms.getMonoIsotopicMass(endgroupPair) for endgroupPair in endgroupPairs)) - abs(int(charge / adductIon[1])) * pymacroms.getMonoIsotopicMass(adductIon[0])) / min(list(pymacroms.getMonoIsotopicMass(monomer) for monomer in monomers))) + 1
+                print("--> maximum sum of repeating units: %i (estimated based on the provided mzRange and the masses of the components)" % maxSumRepeatingUnits)
+            elif minSumRepeatingUnits is None:
+                if mzRange is not None:
+                    minSumRepeatingUnits = max(int((mzRange[0] * abs(charge) - max(list(pymacroms.getMonoIsotopicMass(endgroupPair) for endgroupPair in endgroupPairs)) - abs(int(charge / adductIon[1])) * pymacroms.getMonoIsotopicMass(adductIon[0])) / max(list(pymacroms.getMonoIsotopicMass(monomer) for monomer in monomers))), 0)
+                    print("--> minimum sum of repeating units: %i (estimated based on the provided mzRange and the masses of the components)" % minSumRepeatingUnits)
+                else:
+                    minSumRepeatingUnits = 0
+                    print("--> minimum sum of repeating units set to 0 since it cannot be estimated")
+            # Now build all the combinations of repeating units within the limits of the given (or estimated) parameters
+            RUCombinations = self.getRUCombinations(len(monomers),
+                                                    minRepeatingUnits=minRepeatingUnits,
+                                                    minSumRepeatingUnits=minSumRepeatingUnits,
+                                                    maxRepeatingUnits=maxRepeatingUnits,
+                                                    maxSumRepeatingUnits=maxSumRepeatingUnits)
+            if RUCombinations is None:
+                sys.exit("Failed to build a matrix of possible combinations of monomers based on \nthe provided combination of mzRange, min(Sum)RepeatingUnits and max(Sum)RepeatingUnits.\nPlease check the parameters.")
+            # Finally, combine the combinations of repeating units with the different indices of the different end groups
+            compositions = itertools.product(range(len(endgroupPairs)), RUCombinations)
+            # Dropped the conversion from itertools.product to list as this gave a serious computational penalty and later it doesn't matter for the iteration
+            # itertools doesn't have a length attribute though so output had to be adjusted as well
+            # print("--> Possible combinations of repeating units/end groups based on given parameters: " + str(len(compositions)))
+            print("--> Possible combinations of repeating units/end groups determined")
         print("--> Filtering and calculating the isotopic patterns...")
 
-        formulaDatabase = [[],[]]
+        formulaDatabase = [[], []]
         macromolecules = []
         isobaricSpecies = False
         numberCompositions = 0
+        bar = progressbar.ProgressBar(widgets=[progressbar.Counter(), ' - ', progressbar.Timer()], max_value=progressbar.UnknownLength, prefix='Number of macromolecules assessed: ')
         for EGPairIndex, RUCombination in compositions:
             numberCompositions += 1
             sumRUCombination = sum(RUCombination)
-            if sumRUCombination < minRepeatingUnits or sumRUCombination > maxRepeatingUnits: # kick out the monomer combinations that result in a total amount of repeating units outside the defined range
-                continue
+            # should be redundant as this is now handled by the getRUCombinations function
+            # if sumRUCombination < minRepeatingUnits or sumRUCombination > maxRepeatingUnits: # kick out the monomer combinations that result in a total amount of repeating units outside the defined range
+            #     continue
             if firstMonomerMajor and RUCombination[0] < (sumRUCombination - RUCombination[0])/comonomersToFirstMaxRatio:
                 continue
             lenRUCombination = len(RUCombination)
@@ -225,10 +276,11 @@ class Polymer:
             formulaCounter = endgroupPairs[EGPairIndex] + formulaCounter
             if formulaCounter in formulaDatabase[0]:
                 formulaDatabase[1][formulaDatabase[0].index(formulaCounter)].append([RUCombination, EGPairIndex])
+                macromolecules[formulaDatabase[0].index(formulaCounter)]["isobaricCompositions"].append([EGPairIndex, RUCombination])
                 isobaricSpecies = True
             else:
                 if mzRange is not None:
-                    # Don't even calculate the molecule if it's +- 10 outside the defined mass range
+                    # Don't even calculate the molecule if its monoisotopic mass is +- 10 outside the defined mass range
                     if not mzRange[0] - 10 <= pymacroms.getMonoIsotopicMass(formulaCounter + pymacroms.getCounterFromFormula(self.adductIon)) <= mzRange[1] + 10:
                         continue
                     else:
@@ -238,14 +290,26 @@ class Polymer:
                         continue
                 else:
                     tempMolecule = pymacroms.Molecule(formulaCounter, self.adductIon, charge, minRelAbundance)
+                macromolecules.append({#"id": numberCompositions - 1,
+                                       "repUnitsTotal": sum(RUCombination),
+                                       "repUnitsCombination": RUCombination,
+                                       "endgroupPairIndex": EGPairIndex,
+                                       "isobaricCompositions": [[EGPairIndex, RUCombination]],
+                                       "moleculeData": tempMolecule})
                 formulaDatabase[0].append(formulaCounter)
                 formulaDatabase[1].append([[RUCombination, EGPairIndex]])
-                macromolecules.append({"repUnitsTotal": sum(RUCombination), "repUnitsCombination": RUCombination, "endgroupPairIndex": EGPairIndex, "moleculeData": tempMolecule})
+            bar.update(numberCompositions)
+        bar.update(numberCompositions)
+        bar.finish()
         if isobaricSpecies:
             self.isobaricSpecies = []
             for index, [counter, data] in enumerate(zip(formulaDatabase[0], formulaDatabase[1])):
                 if len(data) > 1:
-                    self.isobaricSpecies.append({"repUnitsCombinations": list(zip(*data))[0], "endgroupPairIndices": list(zip(*data))[1], "moleculeData": macromolecules[index]["moleculeData"]})
+                    self.isobaricSpecies.append({"repUnitsCombinations": list(zip(*data))[0],
+                                                 "endgroupPairIndices": list(zip(*data))[1],
+                                                 "moleculeData": macromolecules[index]["moleculeData"],
+                                                 # "macromoleculeID": macromolecules[index]["id"]
+                                                 })
         else:
             self.isobaricSpecies = None
         print("--> Based on the restrictions, {} (non-isobaric) macromolecules were retained \n    of the {} combinations of repeating units/end groups...".format(len(macromolecules), numberCompositions))
@@ -261,25 +325,40 @@ class Polymer:
             if self.macromolecules[0]["moleculeData"].resolution == resolution:
                 print("--> Isotopologues have already been aggregated with this resolution")
                 return
+        for macromolecule in progressbar.progressbar(self.macromolecules, 0, len(self.macromolecules)):
+            macromolecule["moleculeData"].aggregateIsotopicDist(resolution)
+        '''
         for macromolecule in progressbar.progressbar(self.macromolecules, 0, len(self.macromolecules)):
             macromolecule["moleculeData"].resolution = resolution
             macromolecule["moleculeData"].isotopicDist_resolution = pymacroms.combineIsotopes(macromolecule["moleculeData"].isotopicDist, resolution)
             macromolecule["moleculeData"].massMostAbundant = sorted(macromolecule["moleculeData"].isotopicDist_resolution, key=itemgetter(1), reverse=True)[0][0]
+        '''
 
     def getIsotopicDist(self, resolution: float = None):
-        isotopicDist = []
         if resolution is not None:
             self.aggregateIsotopes(resolution)
-        for macromolecule in self.macromolecules:
-            if resolution is None:
-                for mass, normAbund in macromolecule["moleculeData"].isotopicDist:
-                    isotopicDist.append([mass, normAbund])
-            else:
-                for mass, normAbund in macromolecule["moleculeData"].isotopicDist_resolution:
-                    isotopicDist.append([mass, normAbund])
-        return sorted(isotopicDist)
+        isotopicDist = np.concatenate([np.array(macromolecule["moleculeData"].isotopicDist) for macromolecule in
+                                       self.macromolecules]) if resolution is None else np.concatenate(
+            [np.array(macromolecule["moleculeData"].isotopicDist_resolution) for macromolecule in self.macromolecules])
+        isotopicDist = isotopicDist[isotopicDist[:, 0].argsort()]
+        return [(mass, normAbund) for mass, normAbund in isotopicDist]
 
-    def getMoleculeList(self, experimentalMass: float = None, ppmDev: float = 5, resolution: float = None):
+    def getIsotopicDist_indexed(self, resolution: float = None):
+        try:
+            return self.isotopicDist_indexed[resolution]
+        except:
+            if not hasattr(self, "isotopicDist_indexed"):
+                self.isotopicDist_indexed = dict()
+            if resolution is None:
+                self.isotopicDist_indexed[resolution] = np.concatenate(([np.append(np.full((len(macromolecule["moleculeData"].isotopicDist), 1), index), np.array(macromolecule["moleculeData"].isotopicDist), axis=1) for index, macromolecule in enumerate(self.macromolecules)]))
+            else:
+                try:
+                    self.isotopicDist_indexed[resolution] = np.concatenate(([np.append(np.full((len(macromolecule["moleculeData"].isotopicDist_resolution), 1), index), np.array(macromolecule["moleculeData"].isotopicDist_resolution) if macromolecule["moleculeData"].resolution == resolution else np.array(macromolecule["moleculeData"].aggregateIsotopicDist(resolution)), axis=1) for index, macromolecule in enumerate(self.macromolecules)]))
+                except:
+                    self.isotopicDist_indexed[resolution] = np.concatenate(([np.append(np.full((len(macromolecule["moleculeData"].aggregateIsotopicDist(resolution)), 1), index), np.array(macromolecule["moleculeData"].aggregateIsotopicDist(resolution)), axis=1) for index, macromolecule in enumerate(self.macromolecules)]))
+            return self.isotopicDist_indexed[resolution]
+
+    def getMoleculeList_old(self, experimentalMass: float = None, ppmDev: float = 5, resolution: float = None):
         isotopesNearMass = None
         moleculeList = []
         for index, macromolecule in enumerate(self.macromolecules):
@@ -294,6 +373,23 @@ class Polymer:
         else:
             return None
 
+    def getMoleculeList(self, experimentalMass: float = None, ppmDev: float = 5, resolution: float = None):
+        if experimentalMass is None:
+            return [[index, list(macromolecule["repUnitsCombination"]), None] for index, macromolecule in enumerate(self.macromolecules)]
+        moleculeList = []
+        isotopicDist = self.getIsotopicDist_indexed(resolution)
+        deviation = abs((experimentalMass - isotopicDist[:, 1]) / isotopicDist[:, 1] * 1e6)
+        in_limit = np.where(deviation <= ppmDev)
+        isotopicDist = np.append(isotopicDist[in_limit], np.reshape(deviation, (-1, 1))[in_limit], axis=1)
+        for index in np.unique(isotopicDist[:, 0]):
+            moleculeList.append([int(index),
+                                 list(self.macromolecules[int(index)]["repUnitsCombination"]),
+                                 [(isotopeMass, normAbundance, deviation) for index, isotopeMass, normAbundance, deviation in isotopicDist[np.where(isotopicDist[:, 0] == index)]]])
+        if len(moleculeList) > 0:
+            return sorted(moleculeList, key=itemgetter(1))
+        else:
+            return None
+
     # def printMoleculeList(self, experimentalMass: float = None, ppmDev: float = 5):
     #     print("Monomer combination\tFormula (ion)\tMost abundant mass")
     #     # print("\t" + str(self.monomers))
