@@ -25,6 +25,7 @@ from __future__ import absolute_import
 from math import exp, isclose
 from collections import Counter
 import itertools
+import numpy as np
 import pymacroms
 import IsoSpecPy
 import matplotlib.pyplot as plt
@@ -54,7 +55,8 @@ class Molecule:
 
         # adductIon that is passed must be a str that is present in the database
         self.adductIon = adductIon
-        self.charge = charge
+        self.charge = abs(charge)
+        self.minRelAbundance = minRelAbundance
         try:
             adductIon = pymacroms.database.ionising_species[adductIon]  # [formula: Counter, charge: int, mass_correction: float]
         except:
@@ -62,17 +64,22 @@ class Molecule:
         self.amountAdductIons = abs(int(charge / adductIon[1]))
         # add an int amount of ionising species: charge/charge-per-ion
         # again, the internal variable is a counter, the self. variant is a str
-        formula_ion = formula + Counter({element: amount*self.amountAdductIons for element, amount in adductIon[0].items()})
-        self.formula_ion = pymacroms.getFormulaStrFromCounter(formula_ion)
+        self.formula_ion = pymacroms.getFormulaStrFromCounter(formula +
+                                                              Counter({element: amount*self.amountAdductIons for element, amount in adductIon[0].items()}))
+        # include virtual element for mass correction due to gain/loss of electrons
         # Generate the isotopologues using IsoSpecPy
         # IsoSpecPy returns the isotopic distribution as an iterable (mass, 'normalised' abundance (= probability))
         # since IsoSpec generates a list of isotopes with a relative abundance > minimumRelAbundance, further filtering is not necessary
-        isotopologues = IsoSpecPy.IsoThresholdGenerator(formula=self.formula_ion, threshold=minRelAbundance, absolute=False, get_confs=False)
+        isotopologues = IsoSpecPy.IsoThreshold(formula=self.formula_ion + adductIon[2] + '{}'.format('' if adductIon[2] == '' else self.charge),
+                                               charge=self.charge,
+                                               threshold=minRelAbundance,
+                                               absolute=False,
+                                               get_confs=False)
         # total m/z = mass + mass correction (amount of ionising species * correction per ionising specie) divided by charge
-        self.isotopicDist = sorted(list(((mass + self.amountAdductIons * adductIon[2])/charge, prob) for mass, prob in isotopologues))
+        self.isotopicDist = sorted([(mass, prob) for mass, prob in isotopologues])
         self.massMin = self.isotopicDist[0][0]
         self.massMax = self.isotopicDist[len(self.isotopicDist)-1][0]
-        self.massMostAbundant = sorted(self.isotopicDist, key=itemgetter(1), reverse=True)[0][0]
+        self.massMostAbundant = max(self.isotopicDist, key=itemgetter(1))[0]
         self.molecularWeight = pymacroms.getMolecularWeight(pymacroms.getCounterFromFormula(self.formula))
 
     def printIsotopicDist(self):
@@ -111,19 +118,17 @@ class Molecule:
     def getIsotopesNearMass(self, experimentalMass: float, ppmDev: float = 5, resolution: float = None):
         isotopesNearMass = []
         if resolution is None:
-            for isotopeMass, normAbundance in self.isotopicDist:
-                deviation = (experimentalMass - isotopeMass) / isotopeMass * 1e6
-                if abs(deviation) <= ppmDev:
-                    isotopesNearMass.append((isotopeMass, normAbundance, deviation))
+            isotopicDist = np.array(self.isotopicDist)
         else:
-            if not hasattr(self, "resolution"):
-                self.aggregateIsotopicDist(resolution)
-            elif self.resolution != resolution:
-                self.aggregateIsotopicDist(resolution)
-            for isotopeMass, normAbundance in self.isotopicDist_resolution:
-                deviation = (experimentalMass - isotopeMass) / isotopeMass * 1e6
-                if abs(deviation) <= ppmDev:
-                    isotopesNearMass.append((isotopeMass, normAbundance, deviation))
+            try:
+                isotopicDist = np.array(self.isotopicDist_resolution) if self.resolution == resolution else np.array(self.aggregateIsotopicDist(resolution))
+            except:
+                isotopicDist = np.array(self.aggregateIsotopicDist(resolution))
+        deviation = abs((experimentalMass - isotopicDist[:, 0]) / isotopicDist[:, 0] * 1e6)
+        isotopicDist = np.append(isotopicDist, np.reshape(deviation, (-1, 1)), axis=1)
+        in_limit = np.where(deviation <= ppmDev)
+        for isotopeMass, normAbundance, deviation in isotopicDist[in_limit]:
+            isotopesNearMass.append((isotopeMass, normAbundance, deviation))
         if len(isotopesNearMass) > 0:
             return isotopesNearMass
         else:
@@ -132,6 +137,5 @@ class Molecule:
     def aggregateIsotopicDist(self, resolution: float):
         self.resolution = resolution
         self.isotopicDist_resolution = pymacroms.combineIsotopes(self.isotopicDist, resolution)
-        self.massMostAbundant = sorted(self.isotopicDist_resolution, key=itemgetter(1), reverse=True)[0][0]
-
-
+        self.massMostAbundant = max(self.isotopicDist_resolution, key=itemgetter(1))[0]
+        return self.isotopicDist_resolution
