@@ -22,7 +22,7 @@
 #
 
 from __future__ import absolute_import
-from collections import Counter
+from collections import Counter, defaultdict
 import itertools
 import pymacroms
 import matplotlib.pyplot as plt
@@ -190,9 +190,9 @@ class Spectrum:
                     macromolecules.append(macromoleculeIndex)
                     normAbundances.append(sum(list(zip(*isotopesNearMass))[1])) # the normAbundance, i.e. contribution of 1 macromolecule to the experimental peak is the sum of the normalised abundances within the accuracy of ppmDev
                     if macromoleculeIndex in matchedIsotopes:
-                        matchedIsotopes[macromoleculeIndex] += list((mz, isotopeMass, deviation) for isotopeMass, normAbundance, deviation in isotopesNearMass)
+                        matchedIsotopes[macromoleculeIndex] += isotopesNearMass
                     else:
-                        matchedIsotopes[macromoleculeIndex] = list((mz, isotopeMass, deviation) for isotopeMass, normAbundance, deviation in isotopesNearMass)
+                        matchedIsotopes[macromoleculeIndex] = isotopesNearMass
                 theoPeakContributions.append([macromolecules, normAbundances])
                 # identifiedMacromolecules += macromolecules
                 # matchedCalcMass.append(np.average(masses, weights=massWeightingFactors))
@@ -210,6 +210,61 @@ class Spectrum:
         print("--> %i experimental m/z values could not be matched to a calculated m/z" % len(self.peakData[~self.peakData.peakMatched]))
         print("--> Done\n")
 
+    '''
+    Marginal speed improvement and more difficult to read...
+    def matchPeaks_new(self, polymerSimulation):
+        self.polymerSimulation = polymerSimulation
+
+        print("Matching experimental spectrum to peaks of the simulated polymer...")
+        print("(accuracy = " + str(self.ppmDev) + " ppm, resolution = " + str(self.resolution) + ")")
+        if self.resolution is not None: # check if the isotopes have to be aggregated
+            # check if the isotopes have been aggregated yet for the first macromolecule,
+            # if yes then it should have been done for all of them and this step can be skipped
+            if hasattr(self.polymerSimulation.macromolecules[0]["moleculeData"], 'resolution'):
+                if self.polymerSimulation.macromolecules[0]["moleculeData"].resolution == self.resolution:
+                    aggregateIsotopes = False
+                else:
+                    aggregateIsotopes = True
+            else:
+                aggregateIsotopes = True
+        else:
+            aggregateIsotopes = False
+        if aggregateIsotopes:
+            self.polymerSimulation.aggregateIsotopes(self.resolution)
+        print("--> Total amount of experimental peaks: " + str(len(self.peakData)))
+        theoPeakContributions = []  # for every experimental peak, this variable contains the theoretical composition:
+        # [[indices of contributing macromolecules], [corresponding normalised abundances]]
+        # identifiedMacromolecules = []
+        matchedIsotopes = dict()
+        peakMatched = []
+        print("--> Matching...")
+        sys.stdout.flush()
+        for mz, relAbundance in progressbar.progressbar(self.peakData.itertuples(index=False, name=None), 0, len(self.peakData)):
+            moleculeList = self.polymerSimulation.getMoleculeList(mz, self.ppmDev, self.resolution)
+            if moleculeList is not None:
+                for macromoleculeIndex, repUnitsCombination, isotopesNearMass in moleculeList:
+                    if macromoleculeIndex in matchedIsotopes:
+                        matchedIsotopes[macromoleculeIndex] += isotopesNearMass
+                    else:
+                        matchedIsotopes[macromoleculeIndex] = isotopesNearMass
+                theoPeakContributions.append(list(zip(*[(macromoleculeIndex, sum(list(zip(*isotopesNearMass))[1])) for macromoleculeIndex, repUnitsCombination, isotopesNearMass in moleculeList])))
+                # identifiedMacromolecules += macromolecules
+                # matchedCalcMass.append(np.average(masses, weights=massWeightingFactors))
+                peakMatched.append(True)
+            else:
+                # matchedCalcMass.append(None)
+                peakMatched.append(False)
+        self.peakData["peakMatched"] = peakMatched
+        self.theoPeakContributions = theoPeakContributions
+        # self.identifiedMacromolecules = np.unique(identifiedMacromolecules)
+        # self.identifiedMacromolecules = sorted(list(matchedIsotopes.keys()))
+        self.matchedIsotopes = matchedIsotopes
+        print("--> %i peaks successfully matched" % len(self.peakData[self.peakData.peakMatched]))
+        print("--> These peaks are (potentially) originating from %i macromolecules" % len(self.matchedIsotopes))
+        print("--> %i experimental m/z values could not be matched to a calculated m/z" % len(self.peakData[~self.peakData.peakMatched]))
+        print("--> Done\n")
+    '''
+
     def plotMatchPeaksOverview(self, mzRange: list = None, showMatched: bool = True, showNotMatched: bool = True, savePlotAs: str = None, reportOutput: bool = False):
         y_val = 0 if reportOutput else -0.1
         if showMatched:
@@ -262,6 +317,8 @@ class Spectrum:
         # if all the peaks of the theoretical pattern are matched, this sum should be 1;
         # if only 1 minor peak of the isotopic pattern is found in the experimental spectrum, the sum will be low --> probably mismatch
         df.drop(columns=dfColumnsSum.loc[dfColumnsSum.dfSum < minimumIsotopeFraction].index, inplace=True)
+        # list(df.columns) is now essentially a filtered (minIsotopeFraction) list of matched macromolecules
+        # this is equivalent to sorted([index for index, matchedIsotopeFraction in [(molecule, sum(list(zip(*self.matchedIsotopes[molecule]))[1])) for molecule in self.matchedIsotopes] if matchedIsotopeFraction >= minimumIsotopeFraction])
         regression = linear_model.LinearRegression(fit_intercept=False)
         regression.fit(df, target)
         regressionR2 = regression.score(df, target)
@@ -377,18 +434,19 @@ class Spectrum:
         if not hasattr(self, "composition"):
             sys.exit("Calculate the composition first using Spectrum.calcComposition")
         if self.composition is not None:
-            fittedIsotopicDist = pd.DataFrame(columns=["mz", "normAbundance"])
+            # fittedIsotopicDist = pd.DataFrame(columns=["mz", "normAbundance"])
+            fittedIsotopicDist = np.array([])
             for macromolecule, amount in self.composition.itertuples(name=None):
-                for mz, normAbund in self.polymerSimulation.macromolecules[macromolecule]["moleculeData"].isotopicDist:
-                    fittedIsotopicDist = fittedIsotopicDist.append(pd.DataFrame([[mz, normAbund * amount]], columns=["mz", "normAbundance"]), ignore_index=True)
-            fittedIsotopicDist.normAbundance = fittedIsotopicDist.normAbundance / fittedIsotopicDist.normAbundance.sum() # normalise
-            # fittedIsotopicDist["relAbundance"] = fittedIsotopicDist.normAbundance / fittedIsotopicDist.normAbundance.max()
-            fittedIsotopicDist.sort_values("mz", inplace=True)
-            fittedIsotopicDist.reset_index(drop=True, inplace=True)
+                fittedIsotopicDist = np.append(fittedIsotopicDist,
+                                               np.array([[mz, normAbund * amount] for mz, normAbund in self.polymerSimulation.macromolecules[macromolecule]["moleculeData"].isotopicDist]))
+                # for mz, normAbund in self.polymerSimulation.macromolecules[macromolecule]["moleculeData"].isotopicDist:
+                #     fittedIsotopicDist = fittedIsotopicDist.append(pd.DataFrame([[mz, normAbund * amount]], columns=["mz", "normAbundance"]), ignore_index=True)
+            fittedIsotopicDist = np.reshape(fittedIsotopicDist, (-1, 2))
+            fittedIsotopicDist = fittedIsotopicDist[fittedIsotopicDist[:, 0].argsort()] # sort mz
+            fittedIsotopicDist[:, 1] = fittedIsotopicDist[:, 1] / sum(fittedIsotopicDist[:, 1])  # normalise
             if resolution is not None:
-                return pymacroms.combineIsotopes(fittedIsotopicDist, resolution)
-            else:
-                return fittedIsotopicDist
+                fittedIsotopicDist = pymacroms.combineIsotopes(fittedIsotopicDist, resolution)
+            return pd.DataFrame(fittedIsotopicDist, columns=["mz", "normAbundance"])
         else:
             return None
 
